 /*
 * file      : src/draw.c
 * project   : tm
 * with      : Gtk-2
 *
 * copyright : (C) 2012 by Claude BULIN
 *
 * TM - Edition des heures de travail / Edition of working hours
 * GNU General Public License
 * 
 * This file is part of TM.
 * 
 * TM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 * 
 * TM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with TM.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
	#define gettext_noop(x) (x) 
#endif

#include <gtk/gtk.h>
#include <cairo.h>
#include <cairo-pdf.h>
#include <cairo-ps.h>
#include <cairo-xlib.h>
#include <X11/Xlib.h>
#include <string.h>
#include <stdlib.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_tm/lib.h"
#endif

#include "global.h"
#include "configuser.h"
#include "edit_year.h"
#include "draw.h"

typedef struct {
	GtkWidget	*AdrDrawingarea;			// Adresse de la surface
	cairo_surface_t	*image;					// Adresse de l'image
	gint		image_width;				// Width
	gint		image_height;				// Height
	gdouble		scale;					// Pourcentage de surface de l'image
	gdouble		x0;					// Pos X
	gdouble		y0;					// Pos Y
	gboolean	BoolScaleAdjust;			// TRUE = ajustement de l'echelle avec la surface de la fentre
	gboolean	BoolSaveToFile;				// TRUE si sauvegarde vers fichier
	cairo_t		*cr_draw_str;				// 
	void		(*FuncExtern)( void );			// 
	
} VAR_DRAW;

VAR_DRAW VarDraw = {
	NULL,
	NULL,
	0,
	0,
	1.0,
	0.0,
	0.0,
	TRUE,
	FALSE,
	NULL,
	NULL
	};


// La definition SCALE_MIN correspond au pourcentage de representation le plus petit de l'image de fond
// SCALE_MIN doit etre adaptee suivant la taille la plus petite de la fenetre
#define SCALE_MIN		0.20
// Definition d'un millimetre
// Formule employee en PostScript
#define MM(x)(((gdouble)x * 72.0) / 25.4 )



// 
// 
gboolean draw_is_save_to_file( void )
{
	return( VarDraw.BoolSaveToFile );
}
// 
// 
void draw_year( void )
{
	if( NULL != VarDraw.AdrDrawingarea )
		gtk_widget_queue_draw( VarDraw.AdrDrawingarea );
}
// 
// 
void draw_set_adr_call_func_extern( void (*FuncExtern) ( void ))
{
	VarDraw.FuncExtern = FuncExtern;
}
// 
// 
void draw_str( gint p_x, gint p_y, TYPE_COLOR p_typecolor, gint p_size, gchar *p_str )
{
	gdouble	LocateX;
	gdouble	LocateY;
	gdouble	Red;
	gdouble	Green;
	gdouble	Blue;
	
	cairo_save( VarDraw.cr_draw_str );
	cairo_select_font_face( VarDraw.cr_draw_str, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL );
	cairo_set_font_size( VarDraw.cr_draw_str, p_size );
	switch( p_typecolor ) {
	case COLOR_WHITE :
		Red = 1.0; Green = 1.0; Blue = 1.0;
		break;
	case COLOR_BLACK :
		Red = 0.0; Green = 0.0; Blue = 0.0;
		break;
	case COLOR_RED :
		Red = 1.0; Green = 0.0; Blue = 0.0;
		break;
	case COLOR_GREEN :
		Red = 0.0; Green = 1.0; Blue = 0.0;
		break;
	case COLOR_BLUE :
		Red = 0.0; Green = 0.0; Blue = 1.0;
		break;
	case COLOR_YELLOW :
		Red = 1.0; Green = 1.0; Blue = 0.0;
		break;
	case COLOR_GRIS :
		Red = 0.5; Green = 0.5; Blue = 0.5;
		break;
	case COLOR_MAGENTA :
		Red = 0.5; Green = 0.0; Blue = 0.5;
		break;
	case COLOR_ORANGE :
		Red = 1.0; Green = 0.5; Blue = 0.0;
		break;
	case COLOR_BLEU_FONCE :
		Red = 0.0; Green = 0.0; Blue = 0.8;
		break;
	default :
		Red = 0.0; Green = 0.0; Blue = 0.0;
	}
	
	cairo_set_source_rgb( VarDraw.cr_draw_str, Red, Green, Blue );
	
	LocateX = MM(p_x);
	LocateY = MM(p_y);
	cairo_move_to( VarDraw.cr_draw_str, LocateX,  LocateY );
	cairo_show_text( VarDraw.cr_draw_str, p_str );
	cairo_restore( VarDraw.cr_draw_str );
}
// 
// 
void draw_print_zoom_changed( gdouble p_zoom_scale )
{
	gchar	*Str = NULL;
	
	Str = g_strdup_printf( "  %3.0f%%  ", 100.0 * p_zoom_scale );
	gtk_label_set_text( GTK_LABEL(GLADE_GET_OBJECT("label_zoom")), Str );
	// g_print( "scale = %f   %3.0f%%  \n", VarDraw.scale, 100.0 * VarDraw.scale );
	// g_print( "STR[ scale ] = %s\n", Str );
	g_free( Str );
	Str = NULL;
}
// 
// 
void draw_set_size_request( void )
{
	gtk_widget_set_size_request(
			VarDraw.AdrDrawingarea,
			(gint)((gdouble)VarDraw.image_width * VarDraw.scale),
			(gint)((gdouble)VarDraw.image_height * VarDraw.scale) );
}
// 
// 
gdouble draw_get_scale_adjust( void )
{
	gdouble	return_scale = 1.0;
	
	if( TRUE == VarDraw.BoolScaleAdjust ) {
		gdouble	scale_width = 1.0;
		gdouble	scale_height = 1.0;
		gdouble	new_scale = 1.0;
		
		// La definition SCALE_MIN correspond au pourcentage de representation le plus petit de l'image de fond
		// SCALE_MIN doit etre adaptee suivant la taille la plus petite de la fenetre
		if( VarDraw.scale > SCALE_MIN ) {
			VarDraw.scale = SCALE_MIN;
			draw_set_size_request();
		}
		scale_width = (gdouble)VarDraw.AdrDrawingarea->allocation.width / (gdouble)VarDraw.image_width;
		scale_height = (gdouble)VarDraw.AdrDrawingarea->allocation.height / (gdouble)VarDraw.image_height;
		new_scale = MIN(scale_width, scale_height);
		return_scale = new_scale;
	}
	else {
		return_scale = VarDraw.scale;
	}
	draw_print_zoom_changed( return_scale );
	return( return_scale );
}
// 
// 
void draw_set_format( cairo_t *p_cr )
{
	cairo_scale( p_cr, VarDraw.scale, VarDraw.scale );
	cairo_translate( p_cr, VarDraw.x0, VarDraw.y0 );
	cairo_set_source_rgb( p_cr, 0.0, 0.0, 0.0 );
	if( FALSE == VarDraw.BoolSaveToFile ) {

		// CONTOUR DE LA FEUILLE
		// EPAISSEUR
		cairo_set_line_width( p_cr, 0.5 );
		// HAUT
		cairo_move_to( p_cr, 0.2, 0.2 );
		cairo_rel_line_to( p_cr, VarDraw.image_width, 0.2 );
		// GAUCHE
		cairo_move_to( p_cr, 0.2, 0.2 );
		cairo_rel_line_to( p_cr, 0.2,  VarDraw.image_height );
		cairo_stroke ( p_cr );
		// EPAISSEUR
		cairo_set_line_width( p_cr, 1.5 );
		// DROITE
		cairo_move_to( p_cr, VarDraw.image_width -1, 0.2 );
		cairo_rel_line_to( p_cr, 0.2,  VarDraw.image_height );
		// BAS
		cairo_move_to( p_cr, 0.2,  VarDraw.image_height -1 );
		cairo_rel_line_to( p_cr, VarDraw.image_width, 0.2 );
		cairo_stroke ( p_cr );
		// EPAISSEUR
		cairo_set_line_width( p_cr, 0.3 );
	}
	
	// SAVEGADE ADRESSE DE LA SURFACE
	VarDraw.cr_draw_str = p_cr;
	
	// L'AFFICHAGE EST FAIT DEPUIS L'EXTERIEUR
	if( VarDraw.FuncExtern ) (*VarDraw.FuncExtern)();
}
// 
// 
void draw_paint( cairo_surface_t *p_cs )
{
	cairo_t		*cr;
	gint		canvas_w, canvas_h;
	
	if( FALSE == VarDraw.BoolSaveToFile ) {
		cr = gdk_cairo_create( VarDraw.AdrDrawingarea->window );
	}
	else {
		cr = cairo_create( p_cs );
	}
	cairo_save( cr );
	VarDraw.scale = draw_get_scale_adjust();
	canvas_w = VarDraw.AdrDrawingarea->allocation.width;
	canvas_h = VarDraw.AdrDrawingarea->allocation.height;
	VarDraw.x0 = ((canvas_w / VarDraw.scale) - VarDraw.image_width) / 2.0;
	VarDraw.y0 = ((canvas_h / VarDraw.scale) - VarDraw.image_height) / 2.0;

	cairo_scale( cr, VarDraw.scale, VarDraw.scale );
	cairo_translate( cr, VarDraw.x0, VarDraw.y0 );
	cairo_set_source_surface( cr, VarDraw.image, 0.0, 0.0 );
	cairo_paint( cr );

	// PAINT PREVIEW SURFACE: WHITE
	cairo_set_source_rgb( cr, 1.0, 1.0, 1.0 );
	cairo_rectangle( cr,
			 0,
			 0,
			 VarDraw.image_width,
			 VarDraw.image_height
			 );			
	cairo_fill( cr );
			
	cairo_restore (cr);
	draw_set_format( cr );
	cairo_destroy( cr );
}
// 
// 
void on_button_size_moins_clicked( GtkButton *button, gpointer user_data )
{
	if( NULL != VarDraw.AdrDrawingarea ) {
		VarDraw.BoolScaleAdjust = FALSE;
		VarDraw.scale -= 0.2;
		if( VarDraw.scale < 0.2 ) VarDraw.scale = 0.2;
		draw_set_size_request();
		gtk_widget_queue_draw( VarDraw.AdrDrawingarea );
	}
}
// 
// 
void on_button_size_normal_clicked( GtkButton *button, gpointer user_data )
{
	if( NULL != VarDraw.AdrDrawingarea ) {
		VarDraw.BoolScaleAdjust = FALSE;
		VarDraw.scale  = 1.0;	
		draw_set_size_request();
		gtk_widget_queue_draw( VarDraw.AdrDrawingarea );
	}
}
// 
// 
void on_button_size_plus_clicked( GtkButton *button, gpointer user_data )
{
	if( NULL != VarDraw.AdrDrawingarea ) {
		VarDraw.BoolScaleAdjust = FALSE;
		VarDraw.scale += 0.2;
		if( VarDraw.scale > 6.0 ) VarDraw.scale = 6.0;
		draw_set_size_request();
		gtk_widget_queue_draw( VarDraw.AdrDrawingarea );
	}
}
// 
// 
void on_button_size_adjust_clicked( GtkButton *button, gpointer user_data )
{
	if( NULL != VarDraw.AdrDrawingarea ) {
		VarDraw.BoolScaleAdjust = TRUE;
		gtk_widget_queue_draw( VarDraw.AdrDrawingarea );
	}
}
// 
// 
gboolean on_drawingarea_draw_expose_event( GtkWidget *widget, GdkEventExpose *event, gpointer user_data )
{
	draw_paint( NULL );
	return( TRUE );
}
// 
// 
gboolean on_drawingarea_draw_motion_notify_event( GtkWidget *drawingarea, GdkEventMotion *event, gpointer user_data )
{
	return( TRUE );
}
// 
// 
void on_drawingarea_draw_realize( GtkWidget *widget, gpointer user_data )
{
	VarDraw.AdrDrawingarea  = widget;
	VarDraw.scale           = SCALE_MIN;
	VarDraw.BoolSaveToFile  = FALSE;
	VarDraw.image = cairo_image_surface_create(
				CAIRO_FORMAT_ARGB32,
				595,	// A4	width
				842	// A4	height
				);
	VarDraw.image_width  = cairo_image_surface_get_width( VarDraw.image );
	VarDraw.image_height = cairo_image_surface_get_height( VarDraw.image );
}








// SEE:
// http://en.literateprograms.org/Hello_World_%28C,_Cairo%29
// 
void WinExport_write_png( gchar *fname, gint width, gint height )
{
	cairo_surface_t *cs;

	cs = cairo_image_surface_create( CAIRO_FORMAT_ARGB32, width, height );
	draw_paint( cs );
	cairo_surface_write_to_png( cs, fname );
	cairo_surface_destroy( cs );
}
// SEE:
// http://en.literateprograms.org/Hello_World_%28C,_Cairo%29
// 
void WinExport_write_pdf( gchar *fname, gint width, gint height )
{
	cairo_surface_t *cs;

	cs = cairo_pdf_surface_create( fname, width, height );
	draw_paint( cs );
	cairo_surface_destroy( cs );
}
// SEE:
// http://en.literateprograms.org/Hello_World_%28C,_Cairo%29
// 
void WinExport_write_ps( gchar *fname, gint width, gint height )
{
	cairo_surface_t *cs;
	
	cs = cairo_ps_surface_create( fname, width, height );
	draw_paint( cs );
	cairo_surface_destroy( cs );
}
// 
// 
void draw_call( void )
{
	gint		width;
	gint		height;
	gdouble		OldScale = VarDraw.scale;
	gboolean	BoolScaleAdjust = VarDraw.BoolScaleAdjust;
	gchar		*PathNamePDF = NULL;
	gchar		*PathNamePS = NULL;
	gchar		*PathNamePNG = NULL;
	gchar		*PathNameTXT = NULL;
	gint		SaveMonth = EditYear_get_int_month();
	gint		tab_month[] = {
				CHECKBUTTON_JANVIER, CHECKBUTTON_FEVRIER, CHECKBUTTON_MARS,
				CHECKBUTTON_AVRIL, CHECKBUTTON_MAI, CHECKBUTTON_JUIN,
				CHECKBUTTON_JUILLET, CHECKBUTTON_AOUT, CHECKBUTTON_SEPTEMBRE,
				CHECKBUTTON_OCTOBRE, CHECKBUTTON_NOVEMBRE, CHECKBUTTON_DECEMBRE };	
	gint		Cpt;
	
	// TESTS TOUS LES MOIS
	for( Cpt = 0; Cpt <= 11; Cpt ++ ) {
		
		// LE MOIS EST IL EN SELECTION ?
		if( Config.CheckbuttonMonth & tab_month[ Cpt ] ) {
			
			// gboolean gtk_calendar_select_month( GtkCalendar *calendar, guint month, guint year);
			EditYear_set_month( Cpt );
			
			// LES NOMS DES FICHIERS
			PathNamePDF  = g_strdup_printf( "%s/%s.pdf", Config.PathNameFileExport, EditYear_get_month( Cpt ) );
			PathNamePS   = g_strdup_printf( "%s/%s.ps",  Config.PathNameFileExport, EditYear_get_month( Cpt ) );
			PathNamePNG  = g_strdup_printf( "%s/%s.png", Config.PathNameFileExport, EditYear_get_month( Cpt ) );
			PathNameTXT  = g_strdup_printf( "%s/%s.txt", Config.PathNameFileExport, EditYear_get_month( Cpt ) );
	
			VarDraw.BoolScaleAdjust = FALSE;
	
			// 
			// PDF POSTSCRIPT
			// 
			if( Config.CheckExportPrint & CHECKBUTTON_PDF || Config.CheckExportPrint & CHECKBUTTON_PS ) {
				// SCALING
				VarDraw.scale = 1.0;
				draw_set_size_request();
				gtk_widget_queue_draw( VarDraw.AdrDrawingarea );
				while (gtk_events_pending()) gtk_main_iteration();
				width  = VarDraw.AdrDrawingarea->allocation.width;
				height = VarDraw.AdrDrawingarea->allocation.height;
		
				// 
				// PDF
				// 
				VarDraw.BoolSaveToFile  = TRUE;
				if( Config.CheckExportPrint & CHECKBUTTON_PDF ) {
					WinExport_write_pdf( PathNamePDF, width, height );
				}
				// 
				// POSTSCRIPT
				// 
				if( Config.CheckExportPrint & CHECKBUTTON_PS ) {
					WinExport_write_ps( PathNamePS, width, height );
				}
				VarDraw.BoolSaveToFile  = FALSE;
			}
	
			// 
			// PNG
			// 
			if( Config.CheckExportPrint & CHECKBUTTON_PNG ) {
		
				// SCALING
				VarDraw.scale = 2.0;
				draw_set_size_request();
				gtk_widget_queue_draw( VarDraw.AdrDrawingarea );
				while (gtk_events_pending()) gtk_main_iteration();
				width  = VarDraw.AdrDrawingarea->allocation.width;
				height = VarDraw.AdrDrawingarea->allocation.height;
	
				VarDraw.BoolSaveToFile  = TRUE;
				WinExport_write_png( PathNamePNG, width, height );
				VarDraw.BoolSaveToFile  = FALSE;
			}
		
			// 
			// TXT
			// 
			if( Config.CheckExportPrint & CHECKBUTTON_TXT ) {
			}

			g_free( PathNamePDF );	PathNamePDF = NULL;
			g_free( PathNamePS );	PathNamePS  = NULL;
			g_free( PathNamePNG );	PathNamePNG = NULL;
			g_free( PathNameTXT );	PathNameTXT = NULL;
	
		}
	}
	
	// CONTEXT RESTORE
	// RESTITUTION DU JOUR ET DU MOIS EN COURS
	EditYear_set_month( SaveMonth );
	VarDraw.scale           = OldScale;
	VarDraw.BoolScaleAdjust = BoolScaleAdjust;
	draw_set_size_request();
	gtk_widget_queue_draw( VarDraw.AdrDrawingarea );
}














